// NodeInfo.cpp : implementation file
//

#include "stdafx.h"

#ifdef STRICT							// Already defined by stdafx,
#undef STRICT							// so we avoid warning linking msg
#endif
#ifdef _MBCS							// The same as above
#undef _MBCS
#endif

#include "Max.h"						// Include MAX's main header file
#include "PlugTest.h"
#include "NodeInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern	PlugPanel		thePlugPanel;

/////////////////////////////////////////////////////////////////////////////
// NodeInfo dialog


NodeInfo::NodeInfo(CWnd* pParent /*=NULL*/)
	: CDialog(NodeInfo::IDD, pParent)
{
	theNode = NULL;

	//{{AFX_DATA_INIT(NodeInfo)
	m_Name = _T("");
	m_Type = _T("");
	//}}AFX_DATA_INIT
}


void NodeInfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(NodeInfo)
	DDX_Text(pDX, IDC_NAME, m_Name);
	DDX_Text(pDX, IDC_TYPE, m_Type);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(NodeInfo, CDialog)
	//{{AFX_MSG_MAP(NodeInfo)
	ON_BN_CLICKED(IDC_FATHER, OnFather)
	ON_BN_CLICKED(IDC_CHILD, OnChild)
	ON_BN_CLICKED(IDC_PREVIOUS, OnPrevious)
	ON_BN_CLICKED(IDC_NEXT, OnNext)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// NodeInfo message handlers

BOOL NodeInfo::OnInitDialog() 
{
	CDialog::OnInitDialog();

	if (!theNode)		return TRUE;

	UpdateNodeInfo();

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Update node's information
void NodeInfo::UpdateNodeInfo()
{
	SClass_ID		SCID;
	ObjectState		os;

	//Get the Node's Name
	m_Name = theNode->GetName();

	//Retrieve the Node's type at the current frame
	os = theNode->EvalWorldState(thePlugPanel.ip->GetTime());
	if (os.obj){
		SCID = os.obj->SuperClassID();			//Get the Node's SCID
		switch (SCID) {
			case GEOMOBJECT_CLASS_ID :
				m_Type = "Geometric Object";
				break;
			case CAMERA_CLASS_ID :
				m_Type = "Camera";
				break;
			case LIGHT_CLASS_ID:
				m_Type = "Light";
				break;
			case SHAPE_CLASS_ID:
				m_Type = "Shape";
				break;
			case HELPER_CLASS_ID:
				m_Type = "Helper";
				break;
		}
	}

	//Set buttons' default state
	GetDlgItem(IDC_FATHER)->ModifyStyle(WS_DISABLED, NULL);
	GetDlgItem(IDC_CHILD)->ModifyStyle(WS_DISABLED, NULL);
	GetDlgItem(IDC_PREVIOUS)->ModifyStyle(WS_DISABLED, NULL);
	GetDlgItem(IDC_NEXT)->ModifyStyle(WS_DISABLED, NULL);

	//Set the buttons' state
	if (theNode->GetParentNode()->IsRootNode())
		GetDlgItem(IDC_FATHER)->ModifyStyle(NULL, WS_DISABLED);

	if (!theNode->NumberOfChildren())
		GetDlgItem(IDC_CHILD)->ModifyStyle(NULL, WS_DISABLED);

	if (theNode->GetParentNode()->NumberOfChildren() == 1) {
		GetDlgItem(IDC_NEXT)->ModifyStyle(NULL, WS_DISABLED);
		GetDlgItem(IDC_PREVIOUS)->ModifyStyle(NULL, WS_DISABLED);
	}

	int i = GetNodeIndex(theNode);

	if (!i)
		GetDlgItem(IDC_PREVIOUS)->ModifyStyle(NULL, WS_DISABLED);

	if (i == (theNode->GetParentNode()->NumberOfChildren()-1))
		GetDlgItem(IDC_NEXT)->ModifyStyle(NULL, WS_DISABLED);

	UpdateData(FALSE);
	Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
//Find a Node's index
int NodeInfo::GetNodeIndex(INode* node)
{
	INode*	Father = node->GetParentNode();
	int		Nb = Father->NumberOfChildren();

	for (int i=0; i<Nb; i++) {
		if (Father->GetChildNode(i) == node)
			break;
	}

	//Check if the node was found, return -1 if not found
	if (i==Nb)
		return -1;

	return i;
}

/////////////////////////////////////////////////////////////////////////////
// Father's button Handler
void NodeInfo::OnFather() 
{
	theNode = theNode->GetParentNode();
	UpdateNodeInfo();
	
}

/////////////////////////////////////////////////////////////////////////////
// Child's button Handler
void NodeInfo::OnChild() 
{
	theNode = theNode->GetChildNode(0);
	UpdateNodeInfo();
}

/////////////////////////////////////////////////////////////////////////////
// Previous's button Handler
void NodeInfo::OnPrevious() 
{
	theNode = theNode->GetParentNode()->GetChildNode(GetNodeIndex(theNode)-1);
	UpdateNodeInfo();
	
}

/////////////////////////////////////////////////////////////////////////////
// Next's button Handler
void NodeInfo::OnNext() 
{
	theNode = theNode->GetParentNode()->GetChildNode(GetNodeIndex(theNode)+1);
	UpdateNodeInfo();
	
}
